# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBool, StrictStr
from typing import List, Optional
from typing_extensions import Annotated
from rapidata.api_client.models.boost_leaderboard_model import BoostLeaderboardModel
from rapidata.api_client.models.create_leaderboard_model import CreateLeaderboardModel
from rapidata.api_client.models.create_leaderboard_result import CreateLeaderboardResult
from rapidata.api_client.models.get_leaderboard_by_id_result import GetLeaderboardByIdResult
from rapidata.api_client.models.get_standing_by_id_result import GetStandingByIdResult
from rapidata.api_client.models.paged_result_of_leaderboards_query_result import PagedResultOfLeaderboardsQueryResult
from rapidata.api_client.models.paged_result_of_runs_by_leaderboard_result import PagedResultOfRunsByLeaderboardResult
from rapidata.api_client.models.query_model import QueryModel
from rapidata.api_client.models.standings_by_leaderboard_result import StandingsByLeaderboardResult
from rapidata.api_client.models.submit_participant_result import SubmitParticipantResult
from rapidata.api_client.models.update_leaderboard_model import UpdateLeaderboardModel

from rapidata.api_client.api_client import ApiClient, RequestSerialized
from rapidata.api_client.api_response import ApiResponse
from rapidata.api_client.rest import RESTResponseType


class LeaderboardApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def benchmark_standing_leaderboard_id_participant_id_get(
        self,
        leaderboard_id: StrictStr,
        participant_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetStandingByIdResult:
        """Gets a standing by leaderboardId and participantId.


        :param leaderboard_id:  (required)
        :type leaderboard_id: str
        :param participant_id:  (required)
        :type participant_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._benchmark_standing_leaderboard_id_participant_id_get_serialize(
            leaderboard_id=leaderboard_id,
            participant_id=participant_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetStandingByIdResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def benchmark_standing_leaderboard_id_participant_id_get_with_http_info(
        self,
        leaderboard_id: StrictStr,
        participant_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetStandingByIdResult]:
        """Gets a standing by leaderboardId and participantId.


        :param leaderboard_id:  (required)
        :type leaderboard_id: str
        :param participant_id:  (required)
        :type participant_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._benchmark_standing_leaderboard_id_participant_id_get_serialize(
            leaderboard_id=leaderboard_id,
            participant_id=participant_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetStandingByIdResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def benchmark_standing_leaderboard_id_participant_id_get_without_preload_content(
        self,
        leaderboard_id: StrictStr,
        participant_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Gets a standing by leaderboardId and participantId.


        :param leaderboard_id:  (required)
        :type leaderboard_id: str
        :param participant_id:  (required)
        :type participant_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._benchmark_standing_leaderboard_id_participant_id_get_serialize(
            leaderboard_id=leaderboard_id,
            participant_id=participant_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetStandingByIdResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _benchmark_standing_leaderboard_id_participant_id_get_serialize(
        self,
        leaderboard_id,
        participant_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if leaderboard_id is not None:
            _path_params['leaderboardId'] = leaderboard_id
        if participant_id is not None:
            _path_params['participantId'] = participant_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json', 
                    'text/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/benchmark/standing/{leaderboardId}/{participantId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def leaderboard_leaderboard_id_boost_post(
        self,
        leaderboard_id: Annotated[StrictStr, Field(description="the leaderboard that should be boosted")],
        boost_leaderboard_model: BoostLeaderboardModel,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SubmitParticipantResult:
        """Boosts a subset of participants within a leaderboard.


        :param leaderboard_id: the leaderboard that should be boosted (required)
        :type leaderboard_id: str
        :param boost_leaderboard_model:  (required)
        :type boost_leaderboard_model: BoostLeaderboardModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_boost_post_serialize(
            leaderboard_id=leaderboard_id,
            boost_leaderboard_model=boost_leaderboard_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SubmitParticipantResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def leaderboard_leaderboard_id_boost_post_with_http_info(
        self,
        leaderboard_id: Annotated[StrictStr, Field(description="the leaderboard that should be boosted")],
        boost_leaderboard_model: BoostLeaderboardModel,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SubmitParticipantResult]:
        """Boosts a subset of participants within a leaderboard.


        :param leaderboard_id: the leaderboard that should be boosted (required)
        :type leaderboard_id: str
        :param boost_leaderboard_model:  (required)
        :type boost_leaderboard_model: BoostLeaderboardModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_boost_post_serialize(
            leaderboard_id=leaderboard_id,
            boost_leaderboard_model=boost_leaderboard_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SubmitParticipantResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def leaderboard_leaderboard_id_boost_post_without_preload_content(
        self,
        leaderboard_id: Annotated[StrictStr, Field(description="the leaderboard that should be boosted")],
        boost_leaderboard_model: BoostLeaderboardModel,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Boosts a subset of participants within a leaderboard.


        :param leaderboard_id: the leaderboard that should be boosted (required)
        :type leaderboard_id: str
        :param boost_leaderboard_model:  (required)
        :type boost_leaderboard_model: BoostLeaderboardModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_boost_post_serialize(
            leaderboard_id=leaderboard_id,
            boost_leaderboard_model=boost_leaderboard_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SubmitParticipantResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _leaderboard_leaderboard_id_boost_post_serialize(
        self,
        leaderboard_id,
        boost_leaderboard_model,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if leaderboard_id is not None:
            _path_params['leaderboardId'] = leaderboard_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if boost_leaderboard_model is not None:
            _body_params = boost_leaderboard_model


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json', 
                    'text/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json', 
                        'text/json', 
                        'application/*+json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/leaderboard/{leaderboardId}/boost',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def leaderboard_leaderboard_id_delete(
        self,
        leaderboard_id: Annotated[StrictStr, Field(description="The id of the leaderboard that gets deleted")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Deletes a leaderboard by its ID.


        :param leaderboard_id: The id of the leaderboard that gets deleted (required)
        :type leaderboard_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_delete_serialize(
            leaderboard_id=leaderboard_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def leaderboard_leaderboard_id_delete_with_http_info(
        self,
        leaderboard_id: Annotated[StrictStr, Field(description="The id of the leaderboard that gets deleted")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Deletes a leaderboard by its ID.


        :param leaderboard_id: The id of the leaderboard that gets deleted (required)
        :type leaderboard_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_delete_serialize(
            leaderboard_id=leaderboard_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def leaderboard_leaderboard_id_delete_without_preload_content(
        self,
        leaderboard_id: Annotated[StrictStr, Field(description="The id of the leaderboard that gets deleted")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Deletes a leaderboard by its ID.


        :param leaderboard_id: The id of the leaderboard that gets deleted (required)
        :type leaderboard_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_delete_serialize(
            leaderboard_id=leaderboard_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _leaderboard_leaderboard_id_delete_serialize(
        self,
        leaderboard_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if leaderboard_id is not None:
            _path_params['leaderboardId'] = leaderboard_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter




        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/leaderboard/{leaderboardId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def leaderboard_leaderboard_id_get(
        self,
        leaderboard_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetLeaderboardByIdResult:
        """Gets a leaderboard by its ID.


        :param leaderboard_id:  (required)
        :type leaderboard_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_get_serialize(
            leaderboard_id=leaderboard_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetLeaderboardByIdResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def leaderboard_leaderboard_id_get_with_http_info(
        self,
        leaderboard_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetLeaderboardByIdResult]:
        """Gets a leaderboard by its ID.


        :param leaderboard_id:  (required)
        :type leaderboard_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_get_serialize(
            leaderboard_id=leaderboard_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetLeaderboardByIdResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def leaderboard_leaderboard_id_get_without_preload_content(
        self,
        leaderboard_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Gets a leaderboard by its ID.


        :param leaderboard_id:  (required)
        :type leaderboard_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_get_serialize(
            leaderboard_id=leaderboard_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetLeaderboardByIdResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _leaderboard_leaderboard_id_get_serialize(
        self,
        leaderboard_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if leaderboard_id is not None:
            _path_params['leaderboardId'] = leaderboard_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json', 
                    'text/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/leaderboard/{leaderboardId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def leaderboard_leaderboard_id_patch(
        self,
        leaderboard_id: StrictStr,
        update_leaderboard_model: UpdateLeaderboardModel,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Updates the response config of a leaderboard.


        :param leaderboard_id:  (required)
        :type leaderboard_id: str
        :param update_leaderboard_model:  (required)
        :type update_leaderboard_model: UpdateLeaderboardModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_patch_serialize(
            leaderboard_id=leaderboard_id,
            update_leaderboard_model=update_leaderboard_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def leaderboard_leaderboard_id_patch_with_http_info(
        self,
        leaderboard_id: StrictStr,
        update_leaderboard_model: UpdateLeaderboardModel,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Updates the response config of a leaderboard.


        :param leaderboard_id:  (required)
        :type leaderboard_id: str
        :param update_leaderboard_model:  (required)
        :type update_leaderboard_model: UpdateLeaderboardModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_patch_serialize(
            leaderboard_id=leaderboard_id,
            update_leaderboard_model=update_leaderboard_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def leaderboard_leaderboard_id_patch_without_preload_content(
        self,
        leaderboard_id: StrictStr,
        update_leaderboard_model: UpdateLeaderboardModel,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Updates the response config of a leaderboard.


        :param leaderboard_id:  (required)
        :type leaderboard_id: str
        :param update_leaderboard_model:  (required)
        :type update_leaderboard_model: UpdateLeaderboardModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_patch_serialize(
            leaderboard_id=leaderboard_id,
            update_leaderboard_model=update_leaderboard_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _leaderboard_leaderboard_id_patch_serialize(
        self,
        leaderboard_id,
        update_leaderboard_model,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if leaderboard_id is not None:
            _path_params['leaderboardId'] = leaderboard_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if update_leaderboard_model is not None:
            _body_params = update_leaderboard_model



        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json', 
                        'text/json', 
                        'application/*+json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='PATCH',
            resource_path='/leaderboard/{leaderboardId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def leaderboard_leaderboard_id_runs_get(
        self,
        leaderboard_id: StrictStr,
        request: Optional[QueryModel] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PagedResultOfRunsByLeaderboardResult:
        """Gets the runs related to a leaderboard


        :param leaderboard_id:  (required)
        :type leaderboard_id: str
        :param request: 
        :type request: QueryModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_runs_get_serialize(
            leaderboard_id=leaderboard_id,
            request=request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultOfRunsByLeaderboardResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def leaderboard_leaderboard_id_runs_get_with_http_info(
        self,
        leaderboard_id: StrictStr,
        request: Optional[QueryModel] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PagedResultOfRunsByLeaderboardResult]:
        """Gets the runs related to a leaderboard


        :param leaderboard_id:  (required)
        :type leaderboard_id: str
        :param request: 
        :type request: QueryModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_runs_get_serialize(
            leaderboard_id=leaderboard_id,
            request=request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultOfRunsByLeaderboardResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def leaderboard_leaderboard_id_runs_get_without_preload_content(
        self,
        leaderboard_id: StrictStr,
        request: Optional[QueryModel] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Gets the runs related to a leaderboard


        :param leaderboard_id:  (required)
        :type leaderboard_id: str
        :param request: 
        :type request: QueryModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_runs_get_serialize(
            leaderboard_id=leaderboard_id,
            request=request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultOfRunsByLeaderboardResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _leaderboard_leaderboard_id_runs_get_serialize(
        self,
        leaderboard_id,
        request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if leaderboard_id is not None:
            _path_params['leaderboardId'] = leaderboard_id
        # process the query parameters
        if request is not None:
            
            _query_params.append(('request', request))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json', 
                    'text/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/leaderboard/{leaderboardId}/runs',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def leaderboard_leaderboard_id_standings_get(
        self,
        leaderboard_id: StrictStr,
        tags: Optional[List[StrictStr]] = None,
        use_weighted_scoring: Optional[StrictBool] = None,
        include_confidence_intervals: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> StandingsByLeaderboardResult:
        """leaderboard_leaderboard_id_standings_get


        :param leaderboard_id: (required)
        :type leaderboard_id: str
        :param tags:
        :type tags: List[str]
        :param use_weighted_scoring:
        :type use_weighted_scoring: bool
        :param include_confidence_intervals:
        :type include_confidence_intervals: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_standings_get_serialize(
            leaderboard_id=leaderboard_id,
            tags=tags,
            use_weighted_scoring=use_weighted_scoring,
            include_confidence_intervals=include_confidence_intervals,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "StandingsByLeaderboardResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def leaderboard_leaderboard_id_standings_get_with_http_info(
        self,
        leaderboard_id: StrictStr,
        tags: Optional[List[StrictStr]] = None,
        use_weighted_scoring: Optional[StrictBool] = None,
        include_confidence_intervals: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[StandingsByLeaderboardResult]:
        """leaderboard_leaderboard_id_standings_get


        :param leaderboard_id: (required)
        :type leaderboard_id: str
        :param tags:
        :type tags: List[str]
        :param use_weighted_scoring:
        :type use_weighted_scoring: bool
        :param include_confidence_intervals:
        :type include_confidence_intervals: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_standings_get_serialize(
            leaderboard_id=leaderboard_id,
            tags=tags,
            use_weighted_scoring=use_weighted_scoring,
            include_confidence_intervals=include_confidence_intervals,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "StandingsByLeaderboardResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def leaderboard_leaderboard_id_standings_get_without_preload_content(
        self,
        leaderboard_id: StrictStr,
        tags: Optional[List[StrictStr]] = None,
        use_weighted_scoring: Optional[StrictBool] = None,
        include_confidence_intervals: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """leaderboard_leaderboard_id_standings_get


        :param leaderboard_id: (required)
        :type leaderboard_id: str
        :param tags:
        :type tags: List[str]
        :param use_weighted_scoring:
        :type use_weighted_scoring: bool
        :param include_confidence_intervals:
        :type include_confidence_intervals: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_leaderboard_id_standings_get_serialize(
            leaderboard_id=leaderboard_id,
            tags=tags,
            use_weighted_scoring=use_weighted_scoring,
            include_confidence_intervals=include_confidence_intervals,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "StandingsByLeaderboardResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _leaderboard_leaderboard_id_standings_get_serialize(
        self,
        leaderboard_id,
        tags,
        use_weighted_scoring,
        include_confidence_intervals,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'tags': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if leaderboard_id is not None:
            _path_params['leaderboardId'] = leaderboard_id
        # process the query parameters
        if tags is not None:
            
            _query_params.append(('tags', tags))
            
        if use_weighted_scoring is not None:
            
            _query_params.append(('useWeightedScoring', use_weighted_scoring))
            
        if include_confidence_intervals is not None:
            
            _query_params.append(('includeConfidenceIntervals', include_confidence_intervals))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json', 
                    'text/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/leaderboard/{leaderboardId}/standings',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def leaderboard_post(
        self,
        create_leaderboard_model: CreateLeaderboardModel,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CreateLeaderboardResult:
        """Creates a new leaderboard with the specified name and criteria.


        :param create_leaderboard_model:  (required)
        :type create_leaderboard_model: CreateLeaderboardModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_post_serialize(
            create_leaderboard_model=create_leaderboard_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CreateLeaderboardResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def leaderboard_post_with_http_info(
        self,
        create_leaderboard_model: CreateLeaderboardModel,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CreateLeaderboardResult]:
        """Creates a new leaderboard with the specified name and criteria.


        :param create_leaderboard_model:  (required)
        :type create_leaderboard_model: CreateLeaderboardModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_post_serialize(
            create_leaderboard_model=create_leaderboard_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CreateLeaderboardResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def leaderboard_post_without_preload_content(
        self,
        create_leaderboard_model: CreateLeaderboardModel,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Creates a new leaderboard with the specified name and criteria.


        :param create_leaderboard_model:  (required)
        :type create_leaderboard_model: CreateLeaderboardModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboard_post_serialize(
            create_leaderboard_model=create_leaderboard_model,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CreateLeaderboardResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _leaderboard_post_serialize(
        self,
        create_leaderboard_model,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if create_leaderboard_model is not None:
            _body_params = create_leaderboard_model


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json', 
                    'text/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json', 
                        'text/json', 
                        'application/*+json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/leaderboard',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def leaderboards_get(
        self,
        request: Annotated[Optional[QueryModel], Field(description="Query parameters")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PagedResultOfLeaderboardsQueryResult:
        """Queries all leaderboards for a specific benchmark.


        :param request: Query parameters
        :type request: QueryModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboards_get_serialize(
            request=request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultOfLeaderboardsQueryResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def leaderboards_get_with_http_info(
        self,
        request: Annotated[Optional[QueryModel], Field(description="Query parameters")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PagedResultOfLeaderboardsQueryResult]:
        """Queries all leaderboards for a specific benchmark.


        :param request: Query parameters
        :type request: QueryModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboards_get_serialize(
            request=request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultOfLeaderboardsQueryResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def leaderboards_get_without_preload_content(
        self,
        request: Annotated[Optional[QueryModel], Field(description="Query parameters")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Queries all leaderboards for a specific benchmark.


        :param request: Query parameters
        :type request: QueryModel
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._leaderboards_get_serialize(
            request=request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultOfLeaderboardsQueryResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _leaderboards_get_serialize(
        self,
        request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if request is not None:
            
            _query_params.append(('request', request))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/plain', 
                    'application/json', 
                    'text/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/leaderboards',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





from __future__ import annotations

from rapidata.rapidata_client.referee._base_referee import Referee
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from rapidata.api_client.models.i_referee_model import IRefereeModel


class NaiveReferee(Referee):
    """A simple referee that completes a task after a fixed number of responses.

    This referee implements a straightforward approach to task completion,
    where the task is considered finished after a predetermined number of
    responses have been made, regardless of the content or quality of those responses.

    Args:
        responses (int, optional): The number of responses required
            to complete the task. Defaults to 10. This is per media item.

    Attributes:
        responses (int): The number of responses required to complete the task.
    """

    def __init__(self, responses: int = 10):
        if responses < 1:
            raise ValueError("The number of responses must be greater than 0.")
        super().__init__()
        self.responses = responses

    def _to_dict(self):
        return {
            "_t": "NaiveRefereeConfig",
            "guessesRequired": self.responses,
        }

    def _to_model(self) -> IRefereeModel:
        from rapidata.api_client.models.i_referee_model_naive_referee_model import (
            IRefereeModelNaiveRefereeModel,
        )
        from rapidata.api_client.models.i_referee_model import IRefereeModel

        return IRefereeModel(
            actual_instance=IRefereeModelNaiveRefereeModel(
                _t="NaiveReferee",
                totalVotes=self.responses,
            )
        )



# coding: utf-8

"""
    Rapidata.Dataset

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from rapidata.api_client.models.compare_workflow_config_pair_maker_config import CompareWorkflowConfigPairMakerConfig
from rapidata.api_client.models.compare_workflow_model1_referee import CompareWorkflowModel1Referee
from rapidata.api_client.models.elo_config import EloConfig
from rapidata.api_client.models.feature_flag import FeatureFlag
from rapidata.api_client.models.grouped_ranking_workflow_config_context_assets_value import GroupedRankingWorkflowConfigContextAssetsValue
from typing import Optional, Set
from typing_extensions import Self

class GroupedRankingWorkflowConfig(BaseModel):
    """
    GroupedRankingWorkflowConfig
    """ # noqa: E501
    t: StrictStr = Field(description="Discriminator value for GroupedRankingWorkflowConfig", alias="_t")
    criteria: StrictStr
    referee: CompareWorkflowModel1Referee
    target_country_codes: List[StrictStr] = Field(alias="targetCountryCodes")
    max_parallelism: StrictInt = Field(alias="maxParallelism")
    contexts: Optional[Dict[str, StrictStr]] = None
    context_assets: Optional[Dict[str, GroupedRankingWorkflowConfigContextAssetsValue]] = Field(default=None, alias="contextAssets")
    elo_config: Optional[EloConfig] = Field(default=None, alias="eloConfig")
    pair_maker_config: Optional[CompareWorkflowConfigPairMakerConfig] = Field(default=None, alias="pairMakerConfig")
    feature_flags: Optional[List[FeatureFlag]] = Field(default=None, alias="featureFlags")
    __properties: ClassVar[List[str]] = ["_t", "criteria", "referee", "targetCountryCodes", "maxParallelism", "contexts", "contextAssets", "eloConfig", "pairMakerConfig", "featureFlags"]

    @field_validator('t')
    def t_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['GroupedRankingWorkflowConfig']):
            raise ValueError("must be one of enum values ('GroupedRankingWorkflowConfig')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GroupedRankingWorkflowConfig from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of referee
        if self.referee:
            _dict['referee'] = self.referee.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each value in context_assets (dict)
        _field_dict = {}
        if self.context_assets:
            for _key_context_assets in self.context_assets:
                if self.context_assets[_key_context_assets]:
                    _field_dict[_key_context_assets] = self.context_assets[_key_context_assets].to_dict()
            _dict['contextAssets'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of elo_config
        if self.elo_config:
            _dict['eloConfig'] = self.elo_config.to_dict()
        # override the default output from pydantic by calling `to_dict()` of pair_maker_config
        if self.pair_maker_config:
            _dict['pairMakerConfig'] = self.pair_maker_config.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in feature_flags (list)
        _items = []
        if self.feature_flags:
            for _item_feature_flags in self.feature_flags:
                if _item_feature_flags:
                    _items.append(_item_feature_flags.to_dict())
            _dict['featureFlags'] = _items
        # set to None if contexts (nullable) is None
        # and model_fields_set contains the field
        if self.contexts is None and "contexts" in self.model_fields_set:
            _dict['contexts'] = None

        # set to None if context_assets (nullable) is None
        # and model_fields_set contains the field
        if self.context_assets is None and "context_assets" in self.model_fields_set:
            _dict['contextAssets'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GroupedRankingWorkflowConfig from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "_t": obj.get("_t") if obj.get("_t") is not None else 'GroupedRankingWorkflowConfig',
            "criteria": obj.get("criteria"),
            "referee": CompareWorkflowModel1Referee.from_dict(obj["referee"]) if obj.get("referee") is not None else None,
            "targetCountryCodes": obj.get("targetCountryCodes"),
            "maxParallelism": obj.get("maxParallelism"),
            "contexts": obj.get("contexts"),
            "contextAssets": dict(
                (_k, GroupedRankingWorkflowConfigContextAssetsValue.from_dict(_v))
                for _k, _v in obj["contextAssets"].items()
            )
            if obj.get("contextAssets") is not None
            else None,
            "eloConfig": EloConfig.from_dict(obj["eloConfig"]) if obj.get("eloConfig") is not None else None,
            "pairMakerConfig": CompareWorkflowConfigPairMakerConfig.from_dict(obj["pairMakerConfig"]) if obj.get("pairMakerConfig") is not None else None,
            "featureFlags": [FeatureFlag.from_dict(_item) for _item in obj["featureFlags"]] if obj.get("featureFlags") is not None else None
        })
        return _obj





from __future__ import annotations

from rapidata.rapidata_client.filter._base_filter import RapidataFilter
from rapidata.rapidata_client.filter.models.gender import Gender
from pydantic import BaseModel, ConfigDict


class GenderFilter(RapidataFilter, BaseModel):
    """GenderFilter Class

    Can be used to filter who to target based on their gender.


    Args:
        genders (list[Gender]): List of genders to filter by."""

    model_config = ConfigDict(arbitrary_types_allowed=True)

    genders: list[Gender]

    def __init__(self, genders: list[Gender]):
        super().__init__(genders=genders)

    def _to_model(self):
        from rapidata.api_client.models.i_user_filter_model import IUserFilterModel
        from rapidata.api_client.models.i_user_filter_model_gender_user_filter_model import (
            IUserFilterModelGenderUserFilterModel,
        )

        return IUserFilterModel(
            actual_instance=IUserFilterModelGenderUserFilterModel(
                _t="GenderFilter",
                genders=[gender._to_backend_model() for gender in self.genders],
            )
        )



# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from rapidata.api_client.models.i_user_filter_model_age_user_filter_model import IUserFilterModelAgeUserFilterModel
from rapidata.api_client.models.i_user_filter_model_campaign_user_filter_model import IUserFilterModelCampaignUserFilterModel
from rapidata.api_client.models.i_user_filter_model_country_user_filter_model import IUserFilterModelCountryUserFilterModel
from rapidata.api_client.models.i_user_filter_model_custom_user_filter_model import IUserFilterModelCustomUserFilterModel
from rapidata.api_client.models.i_user_filter_model_gender_user_filter_model import IUserFilterModelGenderUserFilterModel
from rapidata.api_client.models.i_user_filter_model_language_user_filter_model import IUserFilterModelLanguageUserFilterModel
from rapidata.api_client.models.i_user_filter_model_new_user_filter_model import IUserFilterModelNewUserFilterModel
from rapidata.api_client.models.i_user_filter_model_response_count_user_filter_model import IUserFilterModelResponseCountUserFilterModel
from rapidata.api_client.models.i_user_filter_model_user_score_user_filter_model import IUserFilterModelUserScoreUserFilterModel
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

IUSERFILTERMODEL_ONE_OF_SCHEMAS = ["IUserFilterModelAgeUserFilterModel", "IUserFilterModelAndUserFilterModel", "IUserFilterModelCampaignUserFilterModel", "IUserFilterModelCountryUserFilterModel", "IUserFilterModelCustomUserFilterModel", "IUserFilterModelGenderUserFilterModel", "IUserFilterModelLanguageUserFilterModel", "IUserFilterModelNewUserFilterModel", "IUserFilterModelNotUserFilterModel", "IUserFilterModelOrUserFilterModel", "IUserFilterModelResponseCountUserFilterModel", "IUserFilterModelUserScoreUserFilterModel"]

class IUserFilterModel(BaseModel):
    """
    IUserFilterModel
    """
    # data type: IUserFilterModelAgeUserFilterModel
    oneof_schema_1_validator: Optional[IUserFilterModelAgeUserFilterModel] = None
    # data type: IUserFilterModelAndUserFilterModel
    oneof_schema_2_validator: Optional[IUserFilterModelAndUserFilterModel] = None
    # data type: IUserFilterModelCampaignUserFilterModel
    oneof_schema_3_validator: Optional[IUserFilterModelCampaignUserFilterModel] = None
    # data type: IUserFilterModelCountryUserFilterModel
    oneof_schema_4_validator: Optional[IUserFilterModelCountryUserFilterModel] = None
    # data type: IUserFilterModelCustomUserFilterModel
    oneof_schema_5_validator: Optional[IUserFilterModelCustomUserFilterModel] = None
    # data type: IUserFilterModelGenderUserFilterModel
    oneof_schema_6_validator: Optional[IUserFilterModelGenderUserFilterModel] = None
    # data type: IUserFilterModelLanguageUserFilterModel
    oneof_schema_7_validator: Optional[IUserFilterModelLanguageUserFilterModel] = None
    # data type: IUserFilterModelNewUserFilterModel
    oneof_schema_8_validator: Optional[IUserFilterModelNewUserFilterModel] = None
    # data type: IUserFilterModelNotUserFilterModel
    oneof_schema_9_validator: Optional[IUserFilterModelNotUserFilterModel] = None
    # data type: IUserFilterModelOrUserFilterModel
    oneof_schema_10_validator: Optional[IUserFilterModelOrUserFilterModel] = None
    # data type: IUserFilterModelResponseCountUserFilterModel
    oneof_schema_11_validator: Optional[IUserFilterModelResponseCountUserFilterModel] = None
    # data type: IUserFilterModelUserScoreUserFilterModel
    oneof_schema_12_validator: Optional[IUserFilterModelUserScoreUserFilterModel] = None
    actual_instance: Optional[Union[IUserFilterModelAgeUserFilterModel, IUserFilterModelAndUserFilterModel, IUserFilterModelCampaignUserFilterModel, IUserFilterModelCountryUserFilterModel, IUserFilterModelCustomUserFilterModel, IUserFilterModelGenderUserFilterModel, IUserFilterModelLanguageUserFilterModel, IUserFilterModelNewUserFilterModel, IUserFilterModelNotUserFilterModel, IUserFilterModelOrUserFilterModel, IUserFilterModelResponseCountUserFilterModel, IUserFilterModelUserScoreUserFilterModel]] = None
    one_of_schemas: Set[str] = { "IUserFilterModelAgeUserFilterModel", "IUserFilterModelAndUserFilterModel", "IUserFilterModelCampaignUserFilterModel", "IUserFilterModelCountryUserFilterModel", "IUserFilterModelCustomUserFilterModel", "IUserFilterModelGenderUserFilterModel", "IUserFilterModelLanguageUserFilterModel", "IUserFilterModelNewUserFilterModel", "IUserFilterModelNotUserFilterModel", "IUserFilterModelOrUserFilterModel", "IUserFilterModelResponseCountUserFilterModel", "IUserFilterModelUserScoreUserFilterModel" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )


    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        instance = IUserFilterModel.model_construct()
        error_messages = []
        match = 0
        # validate data type: IUserFilterModelAgeUserFilterModel
        if not isinstance(v, IUserFilterModelAgeUserFilterModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IUserFilterModelAgeUserFilterModel`")
        else:
            match += 1
        # validate data type: IUserFilterModelAndUserFilterModel
        if not isinstance(v, IUserFilterModelAndUserFilterModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IUserFilterModelAndUserFilterModel`")
        else:
            match += 1
        # validate data type: IUserFilterModelCampaignUserFilterModel
        if not isinstance(v, IUserFilterModelCampaignUserFilterModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IUserFilterModelCampaignUserFilterModel`")
        else:
            match += 1
        # validate data type: IUserFilterModelCountryUserFilterModel
        if not isinstance(v, IUserFilterModelCountryUserFilterModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IUserFilterModelCountryUserFilterModel`")
        else:
            match += 1
        # validate data type: IUserFilterModelCustomUserFilterModel
        if not isinstance(v, IUserFilterModelCustomUserFilterModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IUserFilterModelCustomUserFilterModel`")
        else:
            match += 1
        # validate data type: IUserFilterModelGenderUserFilterModel
        if not isinstance(v, IUserFilterModelGenderUserFilterModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IUserFilterModelGenderUserFilterModel`")
        else:
            match += 1
        # validate data type: IUserFilterModelLanguageUserFilterModel
        if not isinstance(v, IUserFilterModelLanguageUserFilterModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IUserFilterModelLanguageUserFilterModel`")
        else:
            match += 1
        # validate data type: IUserFilterModelNewUserFilterModel
        if not isinstance(v, IUserFilterModelNewUserFilterModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IUserFilterModelNewUserFilterModel`")
        else:
            match += 1
        # validate data type: IUserFilterModelNotUserFilterModel
        if not isinstance(v, IUserFilterModelNotUserFilterModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IUserFilterModelNotUserFilterModel`")
        else:
            match += 1
        # validate data type: IUserFilterModelOrUserFilterModel
        if not isinstance(v, IUserFilterModelOrUserFilterModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IUserFilterModelOrUserFilterModel`")
        else:
            match += 1
        # validate data type: IUserFilterModelResponseCountUserFilterModel
        if not isinstance(v, IUserFilterModelResponseCountUserFilterModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IUserFilterModelResponseCountUserFilterModel`")
        else:
            match += 1
        # validate data type: IUserFilterModelUserScoreUserFilterModel
        if not isinstance(v, IUserFilterModelUserScoreUserFilterModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IUserFilterModelUserScoreUserFilterModel`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in IUserFilterModel with oneOf schemas: IUserFilterModelAgeUserFilterModel, IUserFilterModelAndUserFilterModel, IUserFilterModelCampaignUserFilterModel, IUserFilterModelCountryUserFilterModel, IUserFilterModelCustomUserFilterModel, IUserFilterModelGenderUserFilterModel, IUserFilterModelLanguageUserFilterModel, IUserFilterModelNewUserFilterModel, IUserFilterModelNotUserFilterModel, IUserFilterModelOrUserFilterModel, IUserFilterModelResponseCountUserFilterModel, IUserFilterModelUserScoreUserFilterModel. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in IUserFilterModel with oneOf schemas: IUserFilterModelAgeUserFilterModel, IUserFilterModelAndUserFilterModel, IUserFilterModelCampaignUserFilterModel, IUserFilterModelCountryUserFilterModel, IUserFilterModelCustomUserFilterModel, IUserFilterModelGenderUserFilterModel, IUserFilterModelLanguageUserFilterModel, IUserFilterModelNewUserFilterModel, IUserFilterModelNotUserFilterModel, IUserFilterModelOrUserFilterModel, IUserFilterModelResponseCountUserFilterModel, IUserFilterModelUserScoreUserFilterModel. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # deserialize data into IUserFilterModelAgeUserFilterModel
        try:
            instance.actual_instance = IUserFilterModelAgeUserFilterModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IUserFilterModelAndUserFilterModel
        try:
            instance.actual_instance = IUserFilterModelAndUserFilterModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IUserFilterModelCampaignUserFilterModel
        try:
            instance.actual_instance = IUserFilterModelCampaignUserFilterModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IUserFilterModelCountryUserFilterModel
        try:
            instance.actual_instance = IUserFilterModelCountryUserFilterModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IUserFilterModelCustomUserFilterModel
        try:
            instance.actual_instance = IUserFilterModelCustomUserFilterModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IUserFilterModelGenderUserFilterModel
        try:
            instance.actual_instance = IUserFilterModelGenderUserFilterModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IUserFilterModelLanguageUserFilterModel
        try:
            instance.actual_instance = IUserFilterModelLanguageUserFilterModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IUserFilterModelNewUserFilterModel
        try:
            instance.actual_instance = IUserFilterModelNewUserFilterModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IUserFilterModelNotUserFilterModel
        try:
            instance.actual_instance = IUserFilterModelNotUserFilterModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IUserFilterModelOrUserFilterModel
        try:
            instance.actual_instance = IUserFilterModelOrUserFilterModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IUserFilterModelResponseCountUserFilterModel
        try:
            instance.actual_instance = IUserFilterModelResponseCountUserFilterModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IUserFilterModelUserScoreUserFilterModel
        try:
            instance.actual_instance = IUserFilterModelUserScoreUserFilterModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into IUserFilterModel with oneOf schemas: IUserFilterModelAgeUserFilterModel, IUserFilterModelAndUserFilterModel, IUserFilterModelCampaignUserFilterModel, IUserFilterModelCountryUserFilterModel, IUserFilterModelCustomUserFilterModel, IUserFilterModelGenderUserFilterModel, IUserFilterModelLanguageUserFilterModel, IUserFilterModelNewUserFilterModel, IUserFilterModelNotUserFilterModel, IUserFilterModelOrUserFilterModel, IUserFilterModelResponseCountUserFilterModel, IUserFilterModelUserScoreUserFilterModel. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into IUserFilterModel with oneOf schemas: IUserFilterModelAgeUserFilterModel, IUserFilterModelAndUserFilterModel, IUserFilterModelCampaignUserFilterModel, IUserFilterModelCountryUserFilterModel, IUserFilterModelCustomUserFilterModel, IUserFilterModelGenderUserFilterModel, IUserFilterModelLanguageUserFilterModel, IUserFilterModelNewUserFilterModel, IUserFilterModelNotUserFilterModel, IUserFilterModelOrUserFilterModel, IUserFilterModelResponseCountUserFilterModel, IUserFilterModelUserScoreUserFilterModel. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], IUserFilterModelAgeUserFilterModel, IUserFilterModelAndUserFilterModel, IUserFilterModelCampaignUserFilterModel, IUserFilterModelCountryUserFilterModel, IUserFilterModelCustomUserFilterModel, IUserFilterModelGenderUserFilterModel, IUserFilterModelLanguageUserFilterModel, IUserFilterModelNewUserFilterModel, IUserFilterModelNotUserFilterModel, IUserFilterModelOrUserFilterModel, IUserFilterModelResponseCountUserFilterModel, IUserFilterModelUserScoreUserFilterModel]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())

from rapidata.api_client.models.i_user_filter_model_and_user_filter_model import IUserFilterModelAndUserFilterModel
from rapidata.api_client.models.i_user_filter_model_not_user_filter_model import IUserFilterModelNotUserFilterModel
from rapidata.api_client.models.i_user_filter_model_or_user_filter_model import IUserFilterModelOrUserFilterModel
# TODO: Rewrite to not use raise_errors
IUserFilterModel.model_rebuild(raise_errors=False)




from rapidata.rapidata_client.settings._rapidata_setting import RapidataSetting
from typing import Literal, Any, Sequence
from pydantic import BaseModel, model_validator, ConfigDict
from rapidata.api_client.models.i_rapid_payload import IRapidPayload


class Rapid(BaseModel):
    asset: str | list[str]
    payload: IRapidPayload
    data_type: Literal["media", "text"] = "media"
    truth: Any | None = None
    context: str | None = None
    media_context: str | None = None
    sentence: str | None = None
    random_correct_probability: float | None = None
    explanation: str | None = None
    settings: Sequence[RapidataSetting] | None = None

    model_config = ConfigDict(
        arbitrary_types_allowed=True, populate_by_name=True, extra="allow"
    )

    @model_validator(mode="after")
    def check_sentence_and_context(self) -> "Rapid":
        if isinstance(self.sentence, str) and isinstance(self.context, str):
            raise ValueError(
                "Both 'sentence' and 'context' cannot be strings at the same time."
            )
        return self



from rapidata.rapidata_client.filter import (
    AgeFilter,
    CountryFilter,
    GenderFilter,
    LanguageFilter,
    UserScoreFilter,
    NotFilter,
    OrFilter,
    AndFilter,
)


class RapidataFilters:
    """RapidataFilters Classes

    These filters can be added to the order to specifically target a certain group of users.

    Note that adding multiple filters to the same order will result in a logical AND operation between the filters.

    Warning:
        This might significantly slow down the number of responses you receive.

    Attributes:
        user_score (UserScoreFilter): Filters for users with a specific user score.
        age (AgeFilter): Filters for users with a specific age.
        country (CountryFilter): Filters for users with a specific country.
        gender (GenderFilter): Filters for users with a specific gender.
        language (LanguageFilter): Filters for users with a specific language.
        not_filter (NotFilter): Inverts the filter.
        or_filter (OrFilter): Combines multiple filters with a logical OR operation.
        and_filter (AndFilter): Combines multiple filters with a logical AND operation.

    Example:
        ```python
        from rapidata import CountryFilter, LanguageFilter
        filters=[CountryFilter(["US", "DE"]), LanguageFilter(["en"])]
        ```

        This ensures the order is only shown to users in the US and Germany whose phones are set to English.

    Info:
        The OR, AND and NOT filter support the |, & and ~ operators respectively.
        The AND is additionally given by the elements in the list.

        ```python
        from rapidata import AgeFilter, LanguageFilter, CountryFilter
        filters=[~AgeFilter([AgeGroup.UNDER_18]), CountryFilter(["US"]) | (CountryFilter(["CA"]) & LanguageFilter(["en"]))]
        ```

        This would return users who are not under 18 years old and are from the US or who are from Canada and whose phones are set to English.
    """

    user_score = UserScoreFilter
    age = AgeFilter
    country = CountryFilter
    gender = GenderFilter
    language = LanguageFilter
    not_filter = NotFilter
    or_filter = OrFilter
    and_filter = AndFilter



from rapidata.rapidata_client.datapoints.metadata._base_metadata import Metadata
from rapidata.api_client.models.prompt_metadata_input import PromptMetadataInput
from pydantic import BaseModel


class PromptMetadata(Metadata, BaseModel):
    """The PromptMetadata class is used to display a prompt to the user."""

    prompt: str

    def to_model(self):
        return PromptMetadataInput(_t="PromptMetadataInput", prompt=self.prompt)



from __future__ import annotations

import time
import urllib.parse
import webbrowser
from colorama import Fore
from typing import Literal, TYPE_CHECKING
from rapidata.api_client import QueryModel
from rapidata.rapidata_client.validation.rapidata_validation_set import (
    RapidataValidationSet,
)
from rapidata.api_client.models.create_validation_set_model import (
    CreateValidationSetModel,
)
from rapidata.service.openapi_service import OpenAPIService
from rapidata.rapidata_client.validation.rapids.rapids_manager import RapidsManager

from rapidata.api_client.models.page_info import PageInfo
from rapidata.api_client.models.root_filter import RootFilter
from rapidata.api_client.models.filter import Filter
from rapidata.api_client.models.sort_criterion import SortCriterion
from rapidata.api_client.models.sort_direction import SortDirection
from rapidata.api_client.models.filter_operator import FilterOperator

from rapidata.rapidata_client.validation.rapids.box import Box

from rapidata.rapidata_client.config import (
    logger,
    managed_print,
    rapidata_config,
    tracer,
)
from tqdm import tqdm
from rapidata.rapidata_client.validation.rapids.rapids import Rapid

if TYPE_CHECKING:
    from rapidata.rapidata_client.workflow import Workflow
    from rapidata.rapidata_client.datapoints._datapoint import Datapoint
    from rapidata.rapidata_client.settings._rapidata_setting import RapidataSetting
    from typing import Sequence


class ValidationSetManager:
    """
    Responsible for everything related to validation sets. From creation to retrieval.

    Attributes:
        rapid (RapidsManager): The RapidsManager instance.
    """

    def __init__(self, openapi_service: OpenAPIService) -> None:
        self._openapi_service = openapi_service
        self.rapid = RapidsManager(openapi_service)
        logger.debug("ValidationSetManager initialized")

    def _get_total_and_labeled_rapids_count(
        self, validation_set_id: str
    ) -> tuple[int, int]:
        uploaded_rapids = self._openapi_service.validation_api.validation_set_validation_set_id_rapids_get(
            validation_set_id=validation_set_id
        ).items
        return len(uploaded_rapids), sum(1 for rapid in uploaded_rapids if rapid.truth)

    def create_classification_set(
        self,
        name: str,
        instruction: str,
        answer_options: list[str],
        datapoints: list[str],
        truths: list[list[str]],
        data_type: Literal["media", "text"] = "media",
        contexts: list[str] | None = None,
        media_contexts: list[str] | None = None,
        explanations: list[str | None] | None = None,
        dimensions: list[str] = [],
    ) -> RapidataValidationSet:
        """Create a classification validation set.

        Args:
            name (str): The name of the validation set. (will not be shown to the labeler)
            instruction (str): The instruction by which the labeler will answer.
            answer_options (list[str]): The options to choose from when answering.
            datapoints (list[str]): The datapoints that will be used for validation.
            truths (list[list[str]]): The truths for each datapoint. Outer list is for each datapoint, inner list is for each truth.\n
                example:
                    options: ["yes", "no", "maybe"]
                    datapoints: ["datapoint1", "datapoint2"]
                    truths: [["yes"], ["no", "maybe"]] -> first datapoint correct answer is "yes", second datapoint is "no" or "maybe"
            data_type (str, optional): The type of data. Defaults to "media" (any form of image, video or audio). Other option: "text".
            contexts (list[str], optional): The contexts for each datapoint. Defaults to None.\n
                If provided has to be the same length as datapoints and will be shown in addition to the instruction and answer options. (Therefore will be different for each datapoint)
                Will be match up with the datapoints using the list index.
            media_contexts (list[str], optional): The list of media contexts i.e. links to the images / videos for the comparison. Defaults to None.\n
                If provided has to be the same length as datapoints and will be shown in addition to the instruction. (Therefore will be different for each datapoint)
                Will be matched up with the datapoints using the list index.
            explanations (list[str | None], optional): The explanations for each datapoint. Will be given to the annotators in case the answer is wrong. Defaults to None.
            dimensions (list[str], optional): The dimensions to add to the validation set accross which users will be tracked. Defaults to [] which is the default dimension.

        Example:
            ```python
            options: ["yes", "no", "maybe"]
            datapoints: ["datapoint1", "datapoint2"]
            truths: [["yes"], ["no", "maybe"]]
            ```
            This would mean: first datapoint correct answer is "yes", second datapoint is "no" or "maybe"
        """
        with tracer.start_as_current_span(
            "ValidationSetManager.create_classification_set"
        ):
            if not datapoints:
                raise ValueError("Datapoints cannot be empty")

            if len(datapoints) != len(truths):
                raise ValueError("The number of datapoints and truths must be equal")

            if not all([isinstance(truth, (list, tuple)) for truth in truths]):
                raise ValueError("Truths must be a list of lists or tuples")

            if contexts and len(contexts) != len(datapoints):
                raise ValueError("The number of contexts and datapoints must be equal")

            if media_contexts and len(media_contexts) != len(datapoints):
                raise ValueError(
                    "The number of media contexts and datapoints must be equal"
                )

            if explanations and len(explanations) != len(datapoints):
                raise ValueError(
                    "The number of explanations and datapoints must be equal, the index must align, but can be padded with None"
                )

            logger.debug("Creating classification rapids")
            rapids: list[Rapid] = []
            for i in range(len(datapoints)):
                rapids.append(
                    self.rapid.classification_rapid(
                        instruction=instruction,
                        answer_options=answer_options,
                        datapoint=datapoints[i],
                        truths=truths[i],
                        data_type=data_type,
                        context=contexts[i] if contexts != None else None,
                        media_context=(
                            media_contexts[i] if media_contexts != None else None
                        ),
                        explanation=explanations[i] if explanations != None else None,
                    )
                )

            logger.debug("Submitting classification rapids")
            return self._submit(name=name, rapids=rapids, dimensions=dimensions)

    def create_compare_set(
        self,
        name: str,
        instruction: str,
        datapoints: list[list[str]],
        truths: list[str],
        data_type: Literal["media", "text"] = "media",
        contexts: list[str] | None = None,
        media_contexts: list[str] | None = None,
        explanation: list[str | None] | None = None,
        dimensions: list[str] = [],
    ) -> RapidataValidationSet:
        """Create a comparison validation set.

        Args:
            name (str): The name of the validation set. (will not be shown to the labeler)
            instruction (str): The instruction to compare against.
            truths (list[str]): The truths for each comparison. List is for each comparison.\n
                example:
                    instruction: "Which image has a cat?"
                    datapoints = [["image1.jpg", "image2.jpg"], ["image3.jpg", "image4.jpg"]]
                    truths: ["image1.jpg", "image4.jpg"] -> first comparison image1.jpg has a cat, second comparison image4.jpg has a cat
            datapoints (list[list[str]]): The compare datapoints to create the validation set with.
                Outer list is for each comparison, inner list the two images/texts that will be compared.
            data_type (str, optional): The type of data. Defaults to "media" (any form of image, video or audio). Other option: "text".
            contexts (list[str], optional): The contexts for each datapoint. Defaults to None.\n
                If provided has to be the same length as datapoints and will be shown in addition to the instruction and truth. (Therefore will be different for each datapoint)
                Will be match up with the datapoints using the list index.
            media_contexts (list[str], optional): The list of media contexts i.e. links to the images / videos for the comparison. Defaults to None.\n
                If provided has to be the same length as datapoints and will be shown in addition to the instruction. (Therefore will be different for each datapoint)
                Will be matched up with the datapoints using the list index.
            explanation (list[str | None], optional): The explanations for each datapoint. Will be given to the annotators in case the answer is wrong. Defaults to None.
            dimensions (list[str], optional): The dimensions to add to the validation set accross which users will be tracked. Defaults to [] which is the default dimension.

        Example:
            ```python
            instruction: "Which image has a cat?"
            datapoints = [["image1.jpg", "image2.jpg"], ["image3.jpg", "image4.jpg"]]
            truths: ["image1.jpg", "image4.jpg"]
            ```
            This would mean: first comparison image1.jpg has a cat, second comparison image4.jpg has a cat
        """
        with tracer.start_as_current_span("ValidationSetManager.create_compare_set"):
            if not datapoints:
                raise ValueError("Datapoints cannot be empty")

            if len(datapoints) != len(truths):
                raise ValueError("The number of datapoints and truths must be equal")

            if not all([isinstance(truth, str) for truth in truths]):
                raise ValueError("Truths must be a list of strings")

            if contexts and len(contexts) != len(datapoints):
                raise ValueError("The number of contexts and datapoints must be equal")

            if media_contexts and len(media_contexts) != len(datapoints):
                raise ValueError(
                    "The number of media contexts and datapoints must be equal"
                )

            if explanation and len(explanation) != len(datapoints):
                raise ValueError(
                    "The number of explanations and datapoints must be equal, the index must align, but can be padded with None"
                )

            logger.debug("Creating comparison rapids")
            rapids: list[Rapid] = []
            for i in range(len(datapoints)):
                rapids.append(
                    self.rapid.compare_rapid(
                        instruction=instruction,
                        truth=truths[i],
                        datapoint=datapoints[i],
                        data_type=data_type,
                        context=contexts[i] if contexts != None else None,
                        media_context=(
                            media_contexts[i] if media_contexts != None else None
                        ),
                        explanation=explanation[i] if explanation != None else None,
                    )
                )

            logger.debug("Submitting comparison rapids")
            return self._submit(name=name, rapids=rapids, dimensions=dimensions)

    def create_select_words_set(
        self,
        name: str,
        instruction: str,
        truths: list[list[int]],
        datapoints: list[str],
        sentences: list[str],
        required_precision: float = 1.0,
        required_completeness: float = 1.0,
        explanation: list[str | None] | None = None,
        dimensions: list[str] = [],
    ) -> RapidataValidationSet:
        """Create a select words validation set.

        Args:
            name (str): The name of the validation set. (will not be shown to the labeler)
            instruction (str): The instruction to show to the labeler.
            truths (list[list[int]]): The truths for each datapoint. Outer list is for each datapoint, inner list is for each truth.\n
                example:
                    datapoints: ["datapoint1", "datapoint2"]
                    sentences: ["this example 1", "this example with another text"]
                    truths: [[0, 1], [2]] -> first datapoint correct words are "this" and "example", second datapoint is "with"
            datapoints (list[str]): The datapoints that will be used for validation.
            sentences (list[str]): The sentences that will be used for validation. The sentece will be split up by spaces to be selected by the labeler.
                Must be the same length as datapoints.
            required_precision (float, optional): The required precision for the labeler to get the rapid correct (minimum ratio of the words selected that need to be correct). Defaults to 1.0 (no wrong word can be selected).
            required_completeness (float, optional): The required completeness for the labeler to get the rapid correct (miminum ratio of total correct words selected). Defaults to 1.0 (all correct words need to be selected).
            explanation (list[str | None], optional): The explanations for each datapoint. Will be given to the annotators in case the answer is wrong. Defaults to None.
            dimensions (list[str], optional): The dimensions to add to the validation set accross which users will be tracked. Defaults to [] which is the default dimension.

        Example:
            ```python
            datapoints: ["datapoint1", "datapoint2"]
            sentences: ["this example 1", "this example with another text"]
            truths: [[0, 1], [2]]
            ```
            This would mean: first datapoint the correct words are "this" and "example", second datapoint is "with"
        """
        with tracer.start_as_current_span(
            "ValidationSetManager.create_select_words_set"
        ):
            if not datapoints:
                raise ValueError("Datapoints cannot be empty")

            if not all([isinstance(truth, (list, tuple)) for truth in truths]):
                raise ValueError("Truths must be a list of lists or tuples")

            if len(datapoints) != len(truths) or len(datapoints) != len(sentences):
                raise ValueError(
                    "The number of datapoints, truths, and sentences must be equal"
                )

            if explanation and len(explanation) != len(datapoints):
                raise ValueError(
                    "The number of explanations and datapoints must be equal, the index must align, but can be padded with None"
                )

            logger.debug("Creating select words rapids")
            rapids: list[Rapid] = []
            for i in range(len(datapoints)):
                rapids.append(
                    self.rapid.select_words_rapid(
                        instruction=instruction,
                        truths=truths[i],
                        datapoint=datapoints[i],
                        sentence=sentences[i],
                        required_precision=required_precision,
                        required_completeness=required_completeness,
                        explanation=explanation[i] if explanation != None else None,
                    )
                )

            logger.debug("Submitting select words rapids")
            return self._submit(name=name, rapids=rapids, dimensions=dimensions)

    def create_locate_set(
        self,
        name: str,
        instruction: str,
        truths: list[list[Box]],
        datapoints: list[str],
        contexts: list[str] | None = None,
        media_contexts: list[str] | None = None,
        explanation: list[str | None] | None = None,
        dimensions: list[str] = [],
    ) -> RapidataValidationSet:
        """Create a locate validation set.

        Args:
            name (str): The name of the validation set. (will not be shown to the labeler)
            instruction (str): The instruction to show to the labeler.
            truths (list[list[Box]]): The truths for each datapoint. Outer list is for each datapoint, inner list is for each truth.\n
                example:
                    datapoints: ["datapoint1", "datapoint2"]
                    truths: [[Box(0, 0, 100, 100)], [Box(50, 50, 150, 150)]] -> first datapoint the object is in the top left corner, second datapoint the object is in the center
            datapoints (list[str]): The datapoints that will be used for validation.
            contexts (list[str], optional): The contexts for each datapoint. Defaults to None.
            media_contexts (list[str], optional): The list of media contexts i.e. links to the images / videos for the comparison. Defaults to None.\n
                If provided has to be the same length as datapoints and will be shown in addition to the instruction. (Therefore will be different for each datapoint)
                Will be matched up with the datapoints using the list index.
            explanation (list[str | None], optional): The explanations for each datapoint. Will be given to the annotators in case the answer is wrong. Defaults to None.
            dimensions (list[str], optional): The dimensions to add to the validation set accross which users will be tracked. Defaults to [] which is the default dimension.

        Example:
            ```python
            datapoints: ["datapoint1", "datapoint2"]
            truths: [[Box(0, 0, 100, 100)], [Box(50, 50, 150, 150)]]
            ```
            This would mean: first datapoint the object is in the top left corner, second datapoint the object is in the center
        """
        with tracer.start_as_current_span("ValidationSetManager.create_locate_set"):
            if not datapoints:
                raise ValueError("Datapoints cannot be empty")

            if len(datapoints) != len(truths):
                raise ValueError("The number of datapoints and truths must be equal")

            if not all([isinstance(truth, (list, tuple)) for truth in truths]):
                raise ValueError("Truths must be a list of lists or tuples")

            if contexts and len(contexts) != len(datapoints):
                raise ValueError("The number of contexts and datapoints must be equal")

            if media_contexts and len(media_contexts) != len(datapoints):
                raise ValueError(
                    "The number of media contexts and datapoints must be equal"
                )

            if explanation and len(explanation) != len(datapoints):
                raise ValueError(
                    "The number of explanations and datapoints must be equal, the index must align, but can be padded with None"
                )

            logger.debug("Creating locate rapids")
            rapids = []
            rapids: list[Rapid] = []
            for i in range(len(datapoints)):
                rapids.append(
                    self.rapid.locate_rapid(
                        instruction=instruction,
                        truths=truths[i],
                        datapoint=datapoints[i],
                        context=contexts[i] if contexts != None else None,
                        media_context=(
                            media_contexts[i] if media_contexts != None else None
                        ),
                        explanation=explanation[i] if explanation != None else None,
                    )
                )

            logger.debug("Submitting locate rapids")
            return self._submit(name=name, rapids=rapids, dimensions=dimensions)

    def create_draw_set(
        self,
        name: str,
        instruction: str,
        truths: list[list[Box]],
        datapoints: list[str],
        contexts: list[str] | None = None,
        media_contexts: list[str] | None = None,
        explanation: list[str | None] | None = None,
        dimensions: list[str] = [],
    ) -> RapidataValidationSet:
        """Create a draw validation set.

        Args:
            name (str): The name of the validation set. (will not be shown to the labeler)
            instruction (str): The instruction to show to the labeler.
            truths (list[list[Box]]): The truths for each datapoint. Outer list is for each datapoint, inner list is for each truth.\n
                example:
                    datapoints: ["datapoint1", "datapoint2"]
                    truths: [[Box(0, 0, 100, 100)], [Box(50, 50, 150, 150)]] -> first datapoint the object is in the top left corner, second datapoint the object is in the center
            datapoints (list[str]): The datapoints that will be used for validation.
            contexts (list[str], optional): The contexts for each datapoint. Defaults to None.
            media_contexts (list[str], optional): The list of media contexts i.e. links to the images / videos for the comparison. Defaults to None.\n
                If provided has to be the same length as datapoints and will be shown in addition to the instruction. (Therefore will be different for each datapoint)
                Will be matched up with the datapoints using the list index.
            explanation (list[str | None], optional): The explanations for each datapoint. Will be given to the annotators in case the answer is wrong. Defaults to None.
            dimensions (list[str], optional): The dimensions to add to the validation set accross which users will be tracked. Defaults to [] which is the default dimension.

        Example:
            ```python
            datapoints: ["datapoint1", "datapoint2"]
            truths: [[Box(0, 0, 100, 100)], [Box(50, 50, 150, 150)]]
            ```
            This would mean: first datapoint the object is in the top left corner, second datapoint the object is in the center
        """
        with tracer.start_as_current_span("ValidationSetManager.create_draw_set"):
            if not datapoints:
                raise ValueError("Datapoints cannot be empty")

            if len(datapoints) != len(truths):
                raise ValueError("The number of datapoints and truths must be equal")

            if not all([isinstance(truth, (list, tuple)) for truth in truths]):
                raise ValueError("Truths must be a list of lists or tuples")

            if contexts and len(contexts) != len(datapoints):
                raise ValueError("The number of contexts and datapoints must be equal")

            if media_contexts and len(media_contexts) != len(datapoints):
                raise ValueError(
                    "The number of media contexts and datapoints must be equal"
                )

            if explanation and len(explanation) != len(datapoints):
                raise ValueError(
                    "The number of explanations and datapoints must be equal, the index must align, but can be padded with None"
                )

            logger.debug("Creating draw rapids")
            rapids: list[Rapid] = []
            for i in range(len(datapoints)):
                rapids.append(
                    self.rapid.draw_rapid(
                        instruction=instruction,
                        truths=truths[i],
                        datapoint=datapoints[i],
                        context=contexts[i] if contexts != None else None,
                        media_context=(
                            media_contexts[i] if media_contexts != None else None
                        ),
                        explanation=explanation[i] if explanation != None else None,
                    )
                )

            logger.debug("Submitting draw rapids")
            return self._submit(name=name, rapids=rapids, dimensions=dimensions)

    def create_timestamp_set(
        self,
        name: str,
        instruction: str,
        truths: list[list[tuple[int, int]]],
        datapoints: list[str],
        contexts: list[str] | None = None,
        media_contexts: list[str] | None = None,
        explanation: list[str | None] | None = None,
        dimensions: list[str] = [],
    ) -> RapidataValidationSet:
        """Create a timestamp validation set.

        Args:
            name (str): The name of the validation set. (will not be shown to the labeler)
            instruction (str): The instruction to show to the labeler.
            truths (list[list[tuple[int, int]]]): The truths for each datapoint defined as start and endpoint based on miliseconds.
                Outer list is for each datapoint, inner list is for each truth.\n
                example:
                    datapoints: ["datapoint1", "datapoint2"]
                    truths: [[(0, 10)], [(20, 30)]] -> first datapoint the correct interval is from 0 to 10, second datapoint the correct interval is from 20 to 30
            datapoints (list[str]): The datapoints that will be used for validation.
            contexts (list[str], optional): The contexts for each datapoint. Defaults to None.
            media_contexts (list[str], optional): The list of media contexts i.e. links to the images / videos for the comparison. Defaults to None.\n
                If provided has to be the same length as datapoints and will be shown in addition to the instruction. (Therefore will be different for each datapoint)
                Will be matched up with the datapoints using the list index.
            explanation (list[str | None], optional): The explanations for each datapoint. Will be given to the annotators in case the answer is wrong. Defaults to None.
            dimensions (list[str], optional): The dimensions to add to the validation set accross which users will be tracked. Defaults to [] which is the default dimension.

        Example:
            ```python
            datapoints: ["datapoint1", "datapoint2"]
            truths: [[(0, 10)], [(20, 30)]]
            ```
            This would mean: first datapoint the correct interval is from 0 to 10, second datapoint the correct interval is from 20 to 30
        """
        with tracer.start_as_current_span("ValidationSetManager.create_timestamp_set"):
            if not datapoints:
                raise ValueError("Datapoints cannot be empty")

            if len(datapoints) != len(truths):
                raise ValueError("The number of datapoints and truths must be equal")

            if not all([isinstance(truth, (list, tuple)) for truth in truths]):
                raise ValueError("Truths must be a list of lists or tuples")

            if contexts and len(contexts) != len(datapoints):
                raise ValueError("The number of contexts and datapoints must be equal")

            if media_contexts and len(media_contexts) != len(datapoints):
                raise ValueError(
                    "The number of media contexts and datapoints must be equal"
                )

            if explanation and len(explanation) != len(datapoints):
                raise ValueError(
                    "The number of explanations and datapoints must be equal, the index must align, but can be padded with None"
                )

            logger.debug("Creating timestamp rapids")
            rapids: list[Rapid] = []
            for i in range(len(datapoints)):
                rapids.append(
                    self.rapid.timestamp_rapid(
                        instruction=instruction,
                        truths=truths[i],
                        datapoint=datapoints[i],
                        context=contexts[i] if contexts != None else None,
                        media_context=(
                            media_contexts[i] if media_contexts != None else None
                        ),
                        explanation=explanation[i] if explanation != None else None,
                    )
                )

            logger.debug("Submitting timestamp rapids")
            return self._submit(name=name, rapids=rapids, dimensions=dimensions)

    def create_mixed_set(
        self,
        name: str,
        rapids: list[Rapid],
        dimensions: list[str] = [],
    ) -> RapidataValidationSet:
        """Create a validation set with a list of rapids.

        Args:
            name (str): The name of the validation set. (will not be shown to the labeler)
            rapids (list[Rapid]): The list of rapids to add to the validation set.
            dimensions (list[str], optional): The dimensions to add to the validation set accross which users will be tracked. Defaults to [] which is the default dimension.
        """
        with tracer.start_as_current_span("ValidationSetManager.create_mixed_set"):
            if not rapids:
                raise ValueError("Rapids cannot be empty")

            return self._submit(name=name, rapids=rapids, dimensions=dimensions)

    def _submit(
        self, name: str, rapids: list[Rapid], dimensions: list[str]
    ) -> RapidataValidationSet:
        logger.debug("Creating validation set")
        validation_set_id = (
            self._openapi_service.validation_api.validation_set_post(
                create_validation_set_model=CreateValidationSetModel(name=name)
            )
        ).validation_set_id

        logger.debug("Validation set created with ID: %s", validation_set_id)

        if validation_set_id is None:
            raise ValueError("Failed to create validation set")

        logger.debug("Creating validation set instance")

        validation_set = RapidataValidationSet(
            name=name,
            validation_set_id=validation_set_id,
            dimensions=dimensions,
            openapi_service=self._openapi_service,
        )
        with tracer.start_as_current_span("Adding rapids to validation set"):
            logger.debug("Adding rapids to validation set")
            failed_rapids = []

            progress_bar = tqdm(
                total=len(rapids),
                desc="Uploading validation tasks",
                disable=rapidata_config.logging.silent_mode,
            )

            for rapid in rapids:
                try:
                    validation_set.add_rapid(rapid)
                    progress_bar.update(1)
                except Exception as e:
                    logger.error(
                        "Failed to add rapid %s to validation set.\nError: %s",
                        rapid.asset,
                        str(e),
                    )
                    failed_rapids.append(rapid.asset)

            progress_bar.close()

            if failed_rapids:
                logger.error(
                    "Failed to add %s datapoints to validation set: %s",
                    len(failed_rapids),
                    failed_rapids,
                )
                raise RuntimeError(
                    f"Failed to add {len(failed_rapids)} datapoints to validation set: {failed_rapids}"
                )

        managed_print()
        managed_print(
            f"Validation set '{name}' created with ID {validation_set_id}\n",
            f"Now viewable under: {validation_set.validation_set_details_page}",
            sep="",
        )

        if dimensions:
            validation_set.update_dimensions(dimensions)

        return validation_set

    def _create_order_validation_set(
        self,
        workflow: Workflow,
        name: str,
        datapoints: list[Datapoint],
        required_amount: int,
        settings: Sequence[RapidataSetting] | None = None,
        dimensions: list[str] = [],
    ) -> RapidataValidationSet:
        with tracer.start_as_current_span(
            "ValidationSetManager._create_order_validation_set"
        ):
            rapids: list[Rapid] = []
            for datapoint in workflow._format_datapoints(datapoints):
                rapids.append(
                    Rapid(
                        asset=datapoint.asset,
                        payload=workflow._to_payload(datapoint),
                        context=datapoint.context,
                        media_context=datapoint.media_context,
                        data_type=datapoint.data_type,
                        settings=settings,
                    )
                )
            validation_set = RapidataValidationSet(
                validation_set_id=self._openapi_service.validation_api.validation_set_post(
                    create_validation_set_model=CreateValidationSetModel(name=name)
                ).validation_set_id,
                name=name,
                dimensions=dimensions,
                openapi_service=self._openapi_service,
            )

            managed_print()
            managed_print(
                Fore.YELLOW
                + f"A new validation set was created. Please annotate {required_amount} datapoint{('s' if required_amount != 1 else '')} before the order can run."
                + Fore.RESET
            )

            link = f"https://app.{self._openapi_service.environment}/validation-set/detail/{validation_set.id}/annotate?maxSize={len(datapoints)}&required={required_amount}"
            could_open_browser = webbrowser.open(link)
            if not could_open_browser:
                encoded_url = urllib.parse.quote(link, safe="%/:=&?~#+!$,;'@()*[]")
                managed_print(
                    Fore.RED
                    + f"Please open this URL in your browser to annotate the validation set: '{encoded_url}'"
                    + Fore.RESET
                )
            else:
                managed_print(
                    Fore.YELLOW
                    + f"Please annotate the validation set. \n'{link}'"
                    + Fore.RESET
                )

            with tracer.start_as_current_span("Annotating validation set"):
                progress_bar = tqdm(
                    total=required_amount,
                    desc="Annotate the validation set",
                    disable=rapidata_config.logging.silent_mode,
                )

                rapid_index = 0
                while True:
                    total_rapids, labeled_rapids = (
                        self._get_total_and_labeled_rapids_count(validation_set.id)
                    )

                    progress_bar.n = labeled_rapids
                    progress_bar.refresh()

                    if labeled_rapids >= required_amount:
                        break

                    if total_rapids < required_amount and rapid_index >= len(rapids):
                        managed_print(
                            Fore.RED
                            + f"""Warning: An order can only be started with at least {required_amount} annotated validation tasks. But only {labeled_rapids}/{required_amount} were annotated.
Either add clearer examples or turn off the 'autoValidationSetCreation' with:

from rapidata import rapidata_config
rapidata_config.order.autoValidationSetCreation = False"""
                            + Fore.RESET
                        )
                        raise RuntimeError(
                            f"Not enough rapids annotated. Required: {required_amount}, Annotated: {labeled_rapids}"
                        )

                    if (
                        rapid_index < len(rapids)
                        and total_rapids - labeled_rapids <= required_amount * 2
                    ):
                        validation_set.add_rapid(rapids[rapid_index])
                        rapid_index += 1

                    time.sleep(2)

                progress_bar.close()

            validation_set.update_dimensions(dimensions)

            return validation_set

    def get_validation_set_by_id(self, validation_set_id: str) -> RapidataValidationSet:
        """Get a validation set by ID.

        Args:
            validation_set_id (str): The ID of the validation set.

        Returns:
            RapidataValidationSet: The ValidationSet instance.
        """

        with tracer.start_as_current_span(
            "ValidationSetManager.get_validation_set_by_id"
        ):
            logger.debug("Getting validation set by ID: %s", validation_set_id)
            validation_set = self._openapi_service.validation_api.validation_set_validation_set_id_get(
                validation_set_id=validation_set_id
            )

            return RapidataValidationSet(
                validation_set_id,
                str(validation_set.name),
                validation_set.dimensions,
                self._openapi_service,
            )

    def find_validation_sets(
        self, name: str = "", amount: int = 10
    ) -> list[RapidataValidationSet]:
        """Find validation sets by name.

        Args:
            name (str, optional): The name to search for. Defaults to "" to match with any set.
            amount (int, optional): The amount of validation sets to return. Defaults to 10.

        Returns:
            list[RapidataValidationSet]: The list of validation sets.
        """
        with tracer.start_as_current_span("ValidationSetManager.find_validation_sets"):
            logger.debug(
                "Finding validation sets with name: %s and amount: %s", name, amount
            )

            validation_page_result = (
                self._openapi_service.validation_api.validation_sets_get(
                    QueryModel(
                        page=PageInfo(index=1, size=amount),
                        filter=RootFilter(
                            filters=[
                                Filter(
                                    field="Name",
                                    operator=FilterOperator.CONTAINS,
                                    value=name,
                                )
                            ]
                        ),
                        sortCriteria=[
                            SortCriterion(
                                direction=SortDirection.DESC, propertyName="CreatedAt"
                            )
                        ],
                    )
                )
            )

            logger.debug("Validation sets found: %s", validation_page_result.items)

            validation_sets = [
                self.get_validation_set_by_id(str(validation_set.id))
                for validation_set in validation_page_result.items
            ]
            return validation_sets

    def __str__(self) -> str:
        return "ValidationSetManager"

    def __repr__(self) -> str:
        return self.__str__()



from typing import Any
from rapidata.api_client.models.i_rapid_payload import IRapidPayload
from rapidata.rapidata_client.workflow import Workflow
from rapidata.api_client.models.i_rapid_payload_free_text_payload import (
    IRapidPayloadFreeTextPayload,
)
from rapidata.rapidata_client.datapoints._datapoint import Datapoint
from rapidata.api_client.models.rapid_modality import RapidModality
from rapidata.api_client.models.i_order_workflow_model import IOrderWorkflowModel
from rapidata.api_client.models.i_order_workflow_model_simple_workflow_model import (
    IOrderWorkflowModelSimpleWorkflowModel,
)
from rapidata.api_client.models.i_rapid_blueprint import IRapidBlueprint
from rapidata.api_client.models.i_rapid_blueprint_free_text_rapid_blueprint import (
    IRapidBlueprintFreeTextRapidBlueprint,
)


class FreeTextWorkflow(Workflow):
    """
    A workflow for free text input tasks.

    This class represents a workflow where users can provide free-form text responses
    to a given instruction.

    Attributes:
        _instruction (str): The instruction to be answered with free text.

    Args:
        instruction (str): The instruction to be presented for free text input.
        validation_system_prompt (str): The system prompt to determine if the provided free text response is spam or not.
            Should always specify that the LLM should respond with 'not spam' or 'spam'.
    """

    modality = RapidModality.FREETEXT

    def __init__(self, instruction: str, validation_system_prompt: str | None = None):
        super().__init__(type="SimpleWorkflowConfig")
        self._instruction = instruction
        self._validation_system_prompt = validation_system_prompt

    def _get_instruction(self) -> str:
        return self._instruction

    def _to_dict(self) -> dict[str, Any]:
        return {
            **super()._to_dict(),
            "blueprint": {
                "_t": "FreeTextBlueprint",
                "question": self._instruction,
                "validationSystemPrompt": self._validation_system_prompt,
            },
        }

    def _to_model(self) -> IOrderWorkflowModel:
        blueprint = IRapidBlueprintFreeTextRapidBlueprint(
            _t="FreeTextBlueprint",
            question=self._instruction,
            validationSystemPrompt=self._validation_system_prompt,
        )

        return IOrderWorkflowModel(
            actual_instance=IOrderWorkflowModelSimpleWorkflowModel(
                _t="SimpleWorkflow",
                blueprint=IRapidBlueprint(actual_instance=blueprint),
            )
        )

    def _to_payload(self, datapoint: Datapoint) -> IRapidPayload:
        return IRapidPayload(
            actual_instance=IRapidPayloadFreeTextPayload(
                _t="FreeTextPayload",
                question=self._instruction,
            )
        )

    def __str__(self) -> str:
        return f"FreeTextWorkflow(instruction='{self._instruction}', validation_system_prompt={self._validation_system_prompt})"

    def __repr__(self) -> str:
        return f"FreeTextWorkflow(instruction={self._instruction!r}, validation_system_prompt={self._validation_system_prompt!r})"



from typing import Any
from rapidata.api_client.models.i_order_workflow_model import IOrderWorkflowModel
from rapidata.api_client.models.i_order_workflow_model_simple_workflow_model import (
    IOrderWorkflowModelSimpleWorkflowModel,
)
from rapidata.api_client.models.i_rapid_blueprint_compare_rapid_blueprint import (
    IRapidBlueprintCompareRapidBlueprint,
)
from rapidata.api_client.models.i_rapid_payload import IRapidPayload
from rapidata.rapidata_client.workflow import Workflow
from rapidata.api_client.models.i_rapid_payload_compare_payload import (
    IRapidPayloadComparePayload,
)
from rapidata.rapidata_client.datapoints._datapoint import Datapoint
from rapidata.api_client.models.i_rapid_blueprint import IRapidBlueprint
from rapidata.api_client.models.rapid_modality import RapidModality


class CompareWorkflow(Workflow):
    """
    A workflow for comparison tasks.

    This class represents a comparison workflow where items are compared based on
    specified instruction.

    Attributes:
        _instruction (str): The instruction used for comparison.

    Args:
        instruction (str): The instruction to be used for comparison.
    """

    modality = RapidModality.COMPARE

    def __init__(self, instruction: str, a_b_names: list[str] | None = None):
        super().__init__(type="CompareWorkflowConfig")
        self._instruction = instruction
        self._a_b_names = a_b_names

    def _get_instruction(self) -> str:
        return self._instruction

    def _to_dict(self) -> dict[str, Any]:
        return {
            **super()._to_dict(),
            "criteria": self._instruction,
            "indexIdentifiers": self._a_b_names,
        }

    def _to_model(self) -> IOrderWorkflowModel:
        blueprint = IRapidBlueprintCompareRapidBlueprint(
            _t="CompareBlueprint",
            criteria=self._instruction,
            indexIdentifiers=self._a_b_names,
        )

        return IOrderWorkflowModel(
            actual_instance=IOrderWorkflowModelSimpleWorkflowModel(
                _t="SimpleWorkflow",
                blueprint=IRapidBlueprint(actual_instance=blueprint),
            )
        )

    def _to_payload(self, datapoint: Datapoint) -> IRapidPayload:
        return IRapidPayload(
            actual_instance=IRapidPayloadComparePayload(
                _t="ComparePayload",
                criteria=self._instruction,
            )
        )

    def __str__(self) -> str:
        return f"CompareWorkflow(instruction='{self._instruction}', a_b_names={self._a_b_names})"

    def __repr__(self) -> str:
        return f"CompareWorkflow(instruction={self._instruction!r}, a_b_names={self._a_b_names!r})"
